#ifndef CMA_CONTROL_DEFINED
#define CMA_CONTROL_DEFINED

#include <stdlib.h>
#include <vector>

namespace CMa
{
	/**
	 * A interface through which CMa::Interpeter communicates with its caller.
	 *
	 * @author OM/ Modified by SL
	 *
	 * STATUS OK
	 */
	template<class Word> class Control
	{
	public:
		/**
		 * Called when exception occurs in the CMa. No other function
		 * of this interface will be called after that and interpreter
		 * will stop immediately.
		 *
		 * @author OM
		 */
		virtual void onException(Exception* e)=0;

		/**
		 * Called when CMa completes its execution and the stack contains
		 * exactly one word.
		 *
		 * @author OM
		 */
		virtual void onComplete(Word result)=0;

		/**
		 * Allows controlling execution of CMa::Interpreter. Before the first
		 * instruction is executed, this function is called. Let X > 0 be
		 * the return value. After that exactly X instructions are executed,
		 * and then the process is repeated. If X == 0, interpreation is
		 * stopped and no other function of this interface is called.
		 *
		 * @author OM
		 */
		virtual int onBreak()=0;
		
		/**
		 * Called when instruction READI is executed.
		 *
		 * @return the value input
		 *
		 * @throw Failure it is allowed to throw exception on I/O error
		 *
		 * @authorOM
		 */
		virtual Word onInput() throw (Failure)=0;

		/**
		 * Called when instrucition WRITEI is executed. Writes int into
		 * output.
		 *
		 * @throw Failure it is allowed to throw exception on I/O error
		 *
		 * @param value the value to be output
		 *
		 * @author OM
		 */
		virtual void onOutput_int(Word value) throw (Failure)=0;
		
		/**
		 * Called when instrucition WRITEC is executed. Writes char into
		 * output.
		 *
		 * @throw Failure it is allowed to throw exception on I/O error
		 *
		 * @param value the value to be output
		 *
		 * @author SL
		 */
		virtual void onOutput_char(Word value) throw (Failure)=0;

	};
}

#endif

